
#pragma once

#include <experimental/memory>

#include <VirtualRobot/VirtualRobot.h>

#include <ArmarXCore/core/Component.h>
#include <ArmarXCore/core/services/tasks/PeriodicTask.h>
#include <ArmarXCore/interface/observers/ObserverInterface.h>
#include <ArmarXCore/util/tasks.h>

#include <ArmarXGui/libraries/ArmarXGuiComponentPlugins/LightweightRemoteGuiComponentPlugin.h>

#include "RobotAPI/libraries/armem/client/plugins/ReaderWriterPlugin.h"
#include "RobotAPI/libraries/armem_objects/client/familiar_object_instance/ObjectReader.h"
#include <RobotAPI/interface/armem/mns/MemoryNameSystemInterface.h>
#include <RobotAPI/interface/armem/server/MemoryInterface.h>
#include <RobotAPI/interface/objectpose/FamiliarObjectPoseStorageInterface.h>
#include <RobotAPI/libraries/armem/client/plugins.h>
#include <RobotAPI/libraries/armem/core/Time.h>
#include <RobotAPI/libraries/armem/core/wm/memory_definitions.h>
#include <RobotAPI/libraries/armem_objects/client/articulated_object/ArticulatedObjectReader.h>
#include <RobotAPI/libraries/armem_objects/client/articulated_object/ArticulatedObjectWriter.h>

namespace armarx::familiar_objects
{

    /**
     * @defgroup Component-ExampleClient ExampleClient
     * @ingroup RobotAPI-Components
     * A description of the component ExampleClient.
     *
     * @class ExampleClient
     * @ingroup Component-ExampleClient
     * @brief Brief description of class ExampleClient.
     *
     * Detailed description of class ExampleClient.
     */
    class FamiliarObjectDetectionExample :
        virtual public armarx::Component,
        virtual public armarx::armem::client::ComponentPluginUser
    {
    public:
        FamiliarObjectDetectionExample();

        /// @see armarx::ManagedIceObject::getDefaultName()
        std::string getDefaultName() const override;

    protected:
        armarx::PropertyDefinitionsPtr createPropertyDefinitions() override;

        void onInitComponent() override;
        void onConnectComponent() override;
        void onDisconnectComponent() override;
        void onExitComponent() override;

        void run();

        void storeExemplaryFamiliarObjectInMemory() const;
        void readExemplaryFamiliarObjectFromMemory() const;

    private:
        struct Properties
        {
            std::string robotName;

            armarx::arondto::ObjectID exemplaryFamiliarObjectID;

        } p;

        // Writing familiar object instances to memory
        objpose::FamiliarObjectPoseStorageInterfacePrx familiarObjectPoseStoragePrx;

        // Reading familiar object instances from memory
        std::experimental::observer_ptr<armem::client::plugins::ReaderWriterPlugin<
            armarx::armem::obj::familiar_object_instance::Reader>>
            familiarObjectInstanceReaderPlugin;
    };

} // namespace armarx::familiar_objects
