/*
 * This file is part of ArmarX.
 *
 * ArmarX is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * ArmarX is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 *
 * @package    RobotAPI::ArmarXObjects::MemoryToDebugObserver
 * @author     Rainer Kartmann ( rainer dot kartmann at kit dot edu )
 * @date       2023
 * @copyright  http://www.gnu.org/licenses/gpl-2.0.txt
 *             GNU General Public License
 */

#pragma once

#include <ArmarXCore/core/Component.h>
#include <ArmarXCore/libraries/ArmarXCoreComponentPlugins/DebugObserverComponentPlugin.h>
#include <ArmarXCore/util/tasks.h>

#include <ArmarXGui/libraries/ArmarXGuiComponentPlugins/LightweightRemoteGuiComponentPlugin.h>

#include <RobotAPI/libraries/armem/client/plugins/PluginUser.h>
#include <RobotAPI/libraries/armem/client/util/MemoryToDebugObserver.h>

namespace armarx::memory_to_debug_observer
{

    /**
     * @defgroup Component-MemoryToDebugObserver MemoryToDebugObserver
     * @ingroup RobotAPI-Components
     *
     * Transfers data from the memory system to the \ref Component-DebugObserver "Debug Observer",
     * allowing to visualize them in the \ref ArmarXGui-GuiPlugins-PlotterPlugin "Live Plotter".
     *
     * @see armarx::armem::client::util::MemoryToDebugObserver for the business logic
     * implementation.
     *
     * @class Component
     * @ingroup Component-MemoryToDebugObserver
     * @brief Implementation of \ref Component-MemoryToDebugObserver.
     *
     * @see armarx::armem::client::util::MemoryToDebugObserver for the business logic
     * implementation.
     */
    class Component :
        virtual public armarx::Component,
        virtual public armarx::armem::ClientPluginUser,
        virtual public armarx::DebugObserverComponentPluginUser,
        virtual public armarx::LightweightRemoteGuiComponentPluginUser
    {
    public:
        Component();

        /// @see armarx::ManagedIceObject::getDefaultName()
        std::string getDefaultName() const override;

        // LightweightRemoteGuiComponentPluginUser interface
    public:
        void createRemoteGuiTab();
        void RemoteGui_update() override;


    protected:
        armarx::PropertyDefinitionsPtr createPropertyDefinitions() override;

        void onInitComponent() override;
        void onConnectComponent() override;
        void onDisconnectComponent() override;
        void onExitComponent() override;

        void runLoop();


    private:
        struct Properties
        {
            armem::client::util::MemoryToDebugObserver::Properties memoryToDebugObserver;
            std::string memoryToDebugObserverJson;

            float pollFrequencyHz = 30;
        };

        Properties properties;

        armarx::RunningTask<Component>::pointer_type task;

        struct RemoteGuiTab : RemoteGui::Client::Tab
        {
            std::atomic_bool rebuild = false;

            RemoteGui::Client::GroupBox group;
        };

        RemoteGuiTab tab;
    };
} // namespace armarx::memory_to_debug_observer
