

#include "HelloWorld.h"

#include <RobotAPI/libraries/aron/converter/json/NLohmannJSONConverter.h>
#include <RobotAPI/libraries/aron/core/type/variant/container/Object.h>
#include <RobotAPI/libraries/aron/core/type/variant/primitive/String.h>

namespace armarx::skills::provider
{
    HelloWorldSkill::HelloWorldSkill() :
        SimpleSpecializedSkill<skills::Example::HelloWorldAcceptedType>(GetSkillDescription())
    {
    }

    SkillDescription
    HelloWorldSkill::GetSkillDescription()
    {
        armarx::skills::Example::HelloWorldAcceptedType root_profile_params;
        root_profile_params.some_float = 5;
        root_profile_params.some_int = 42;
        root_profile_params.some_text = "YOLO";
        root_profile_params.some_optional_text = "OPTIONAL";
        root_profile_params.some_list_of_matrices.push_back(Eigen::Matrix3f::Zero());
        root_profile_params.some_matrix = Eigen::Matrix3f::Zero();

        return SkillDescription{.skillId = skills::SkillID{.skillName = "HelloWorld"},
                                .description = "This skill logs a message on ARMARX_IMPORTANT",
                                .rootProfileDefaults = root_profile_params.toAron(),
                                .timeout = armarx::core::time::Duration::MilliSeconds(1000),
                                .parametersType =
                                    armarx::skills::Example::HelloWorldAcceptedType::ToAronType(),
                                .resultType = armarx::skills::Example::HelloWorldAcceptedType::ToAronType()};
    }

    Skill::MainResult
    HelloWorldSkill::main(const SpecializedMainInput& in)
    {
        ARMARX_IMPORTANT << "Hi, from the Hello World Skill.\n"
                         << "I received the following data: \n"
                         << aron::data::converter::AronNlohmannJSONConverter::ConvertToNlohmannJSON(
                                in.parameters.toAron())
                                .dump(2)
                         << "\n"
                         << "(executed at: " << IceUtil::Time::now() << ")";
        return {TerminatedSkillStatus::Succeeded, in.parameters.toAron()};
    }
} // namespace armarx::skills::provider
