/*
 * This file is part of ArmarX.
 *
 * ArmarX is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * ArmarX is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 *
 * @package    RobotAPI::ArmarXObjects::NJointHolonomicPlatformUnitVelocityPassThroughController
 * @author     Raphael Grimm ( raphael dot grimm at kit dot edu )
 * @date       2017
 * @copyright  http://www.gnu.org/licenses/gpl-2.0.txt
 *             GNU General Public License
 */

#pragma once

#include <VirtualRobot/Robot.h>

#include <RobotAPI/components/units/RobotUnit/NJointControllers/NJointHolonomicPlatformVelocityControllerInterface.h>

#include "../ControlTargets/ControlTargetHolonomicPlatformVelocity.h"
#include "../util.h"
#include "NJointControllerWithTripleBuffer.h"

namespace armarx
{

    TYPEDEF_PTRS_HANDLE(NJointHolonomicPlatformUnitVelocityPassThroughControllerConfig);

    class NJointHolonomicPlatformUnitVelocityPassThroughControllerConfig :
        virtual public NJointControllerConfig
    {
    public:
        std::string platformName;
        float initialVelocityX;
        float initialVelocityY;
        float initialVelocityRotation;
    };

    TYPEDEF_PTRS_HANDLE(NJointHolonomicPlatformUnitVelocityPassThroughController);

    /**
     * @brief The NJointHolonomicPlatformUnitVelocityPassThroughController class
     * @ingroup Library-RobotUnit-NJointControllers
     */
    class NJointHolonomicPlatformUnitVelocityPassThroughController :
        virtual public NJointHolonomicPlatformVelocityControllerInterface
    {
    public:
        using ConfigPtrT = NJointHolonomicPlatformUnitVelocityPassThroughControllerConfigPtr;

        NJointHolonomicPlatformUnitVelocityPassThroughController(
            RobotUnit* prov,
            NJointHolonomicPlatformUnitVelocityPassThroughControllerConfigPtr config,
            const VirtualRobot::RobotPtr&);

        void rtRun(const IceUtil::Time&, const IceUtil::Time&) override;

        //ice interface
        std::string
        getClassName(const Ice::Current& = Ice::emptyCurrent) const override
        {
            return "NJointHolonomicPlatformUnitVelocityPassThroughController";
        }

    protected:
        ControlTargetHolonomicPlatformVelocity* target;
        NJointHolonomicPlatformVelocityControllerControlData initialSettings;
    };
} // namespace armarx
