/*
 * This file is part of ArmarX.
 *
 * ArmarX is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * ArmarX is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 *
 * @package    RobotAPI::NJointHolonomicPlatformVelocityControllerInterface
 * @author     Tobias Gröger ( tobias dot groeger at student dot kit dot edu )
 * @date       2024
 * @copyright  http://www.gnu.org/licenses/gpl-2.0.txt
 *             GNU General Public License
 */

#pragma once

#include <VirtualRobot/Robot.h>

#include <RobotAPI/components/units/RobotUnit/NJointControllers/NJointHolonomicPlatformVelocityControllerInterface.h>

#include "../ControlTargets/ControlTargetHolonomicPlatformVelocity.h"
#include "NJointControllerWithTripleBuffer.h"

namespace armarx
{

    class Cartesian2DimVelocityRamp
    {
    public:
        Cartesian2DimVelocityRamp(float maxPositionAcceleration, float maxOrientationAcceleration);
        void init(const Eigen::Vector3f& state);

        Eigen::Vector3f update(const Eigen::Vector3f& target, float dt);

        void setMaxPositionAcceleration(float maxPositionAcceleration);
        void setMaxOrientationAcceleration(float maxOrientationAcceleration);

    private:
        float maxPositionAcceleration = 0;
        float maxOrientationAcceleration = 0;

        Eigen::Vector3f state; // [velX,velY,velRotation]
    };

    TYPEDEF_PTRS_HANDLE(NJointHolonomicPlatformVelocityControllerWithRampConfig);

    class NJointHolonomicPlatformVelocityControllerWithRampConfig :
        virtual public NJointControllerConfig
    {
    public:
        std::string platformName;

        float maxPositionAcceleration;
        float maxOrientationAcceleration;
    };

    TYPEDEF_PTRS_HANDLE(NJointHolonomicPlatformVelocityControllerWithRamp);

    class NJointHolonomicPlatformVelocityControllerWithRamp :
        virtual public NJointHolonomicPlatformVelocityControllerInterface
    {
    public:
        using ConfigPtrT = NJointHolonomicPlatformVelocityControllerWithRampConfigPtr;

        NJointHolonomicPlatformVelocityControllerWithRamp(RobotUnit* prov,
                                                          ConfigPtrT config,
                                                          const VirtualRobot::RobotPtr&);

        void rtRun(const IceUtil::Time& sensorValuesTimestamp,
                   const IceUtil::Time& timeSinceLastIteration) override;


        void setMaxAccelerations(float maxPositionAcceleration, float maxOrientationAcceleration);

        //ice interface
        std::string
        getClassName(const Ice::Current& = Ice::emptyCurrent) const override
        {
            return "NJointHolonomicPlatformVelocityControllerWithRamp";
        }

    protected:
        void rtPreActivateController() override;

    protected:
        Cartesian2DimVelocityRamp ramp;

        Eigen::Vector3f activationVelocity;
        IceUtil::Time activationTime;

        ControlTargetHolonomicPlatformVelocity* target;
        const SensorValueHolonomicPlatformVelocity* velocitySensor;
    };
} // namespace armarx
