#pragma once

#include <string>

#include <RobotAPI/components/units/RobotUnit/SensorValues/SensorValueBase.h>

namespace armarx
{

    enum class BatteryState : std::uint8_t
    {
        unavailable = 0,
        discharging = 1,
        charging = 2,
        notCharging = 3,
        full = 4
    };

    class SensorValueBattery : virtual public SensorValueBase
    {
    public:
        DETAIL_SensorValueBase_DEFAULT_METHOD_IMPLEMENTATION

            std::uint8_t state;
        float designEnergy_Wh;
        float fullChargeEnergy_Wh;
        float energy_Wh;
        float energyFromFullCharge_pct;
        float fullEnergyFromDesignEnergy_pct;
        float temperature_degC;
        float ratedVoltage_V;
        float voltage_V;
        float current_A;
        float power_W;
        float remainingTime_h;
        bool hasError;

        static SensorValueInfo<SensorValueBattery>
        GetClassMemberInfo()
        {
            SensorValueInfo<SensorValueBattery> svi;
            svi.addMemberVariable(&SensorValueBattery::state, "state");
            svi.addMemberVariable(&SensorValueBattery::designEnergy_Wh, "designEnergy_Wh");
            svi.addMemberVariable(&SensorValueBattery::fullChargeEnergy_Wh, "fullChargeEnergy_Wh");
            svi.addMemberVariable(&SensorValueBattery::energy_Wh, "energy_Wh");
            svi.addMemberVariable(&SensorValueBattery::energyFromFullCharge_pct,
                                  "energyFromFullCharge_pct");
            svi.addMemberVariable(&SensorValueBattery::fullEnergyFromDesignEnergy_pct,
                                  "fullEnergyFromDesignEnergy_pct");
            svi.addMemberVariable(&SensorValueBattery::temperature_degC, "temperature_degC");
            svi.addMemberVariable(&SensorValueBattery::ratedVoltage_V, "ratedVoltage_V");
            svi.addMemberVariable(&SensorValueBattery::voltage_V, "voltage_V");
            svi.addMemberVariable(&SensorValueBattery::current_A, "current_A");
            svi.addMemberVariable(&SensorValueBattery::power_W, "power_W");
            svi.addMemberVariable(&SensorValueBattery::remainingTime_h, "remainingTime_h");
            svi.addMemberVariable(&SensorValueBattery::hasError, "hasError");
            return svi;
        }
    };

} // namespace armarx
