#pragma once

#include <QComboBox>

#include "RobotAPI/libraries/aron/core/data/variant/primitive/Int.h"
#include "RobotAPI/libraries/aron/core/type/variant/forward_declarations.h"

#include "../AronTreeWidgetItem.h"
#include "CustomWidget.h"

namespace armarx
{
    // Custom wrapper around the QComboBox widget to represent IntEnums
    // This class handles parsing once the user edited the field and is also responsible
    // to parse the according aron data from text
    class IntEnumWidget : public CustomWidget
    {
        Q_OBJECT
    public:
        IntEnumWidget(const aron::type::IntEnumPtr&, QTreeWidgetItem* currentItem);
        // needs to be called after the constructor, because qt needs the meta object to be fully constructed at this point
        void postCtorCall();

        static IntEnumWidget* DynamicCast(QWidget*);
        static IntEnumWidget* DynamicCastAndCheck(QWidget*);

        bool hasParseErrors();
        void setText(const QString& text);
        QString getText();
        std::pair<bool, aron::data::IntPtr> parseToAron();

    private:
        const aron::type::IntEnumPtr element_type;
        QComboBox* innerWidget;

        bool noParseErrors = true;

        void highlightUnparsableEntries();
    private slots:
        void textChanged(const QString&);
    };
} // namespace armarx
