#pragma once

#include <memory>
#include <optional>
#include <string>

#include <SimoxUtility/shapes/OrientedBox.h>

#include <RobotAPI/libraries/core/FramedPose.h>
#include <RobotAPI/libraries/core/Names.h>

namespace armarx::priorknowledge::util
{
    enum class LocationType
    {
        FRAMED_LOCATION,
        FRAMED_BOXED_LOCATION
    };

    struct LocationId
    {
        std::string source;
        std::string name;

        LocationId(const std::string& s, const std::string& n) : source(s), name(n)
        {
        }

        std::string toString() const;
    };

    struct Location
    {
        LocationId id;
        LocationType type;
        std::optional<Names> names;

        Location(const LocationId& i,
                 const LocationType t,
                 const std::optional<Names>& names = std::nullopt) :
            id(i), type(t), names(names)
        {
        }

        virtual ~Location() = default;
    };

    struct FramedLocation : public Location
    {
        std::string frame;
        std::string agent;
        Eigen::Matrix4f pose;

        FramedLocation(const LocationId& id,
                       const LocationType type,
                       const std::string& frame,
                       const std::string& agent,
                       const Eigen::Matrix4f& pose,
                       const std::optional<Names>& names = std::nullopt) :
            Location(id, type, names), frame(frame), agent(agent), pose(pose)
        {
        }

        virtual ~FramedLocation() = default;

        armarx::FramedPose toFramedPose();
    };

    struct FramedBoxedLocation : public FramedLocation
    {
        Eigen::Vector3f extents;

        FramedBoxedLocation(const LocationId& id,
                            const LocationType type,
                            const std::string& frame,
                            const std::string& agent,
                            const Eigen::Matrix4f& pose,
                            const Eigen::Vector3f& extents,
                            const std::optional<Names>& names = std::nullopt) :
            FramedLocation(id, type, frame, agent, pose, names), extents(extents)
        {
        }

        virtual ~FramedBoxedLocation() = default;

        simox::OrientedBox<float> toOrientedBox();
    };

    using FramedBoxedLocationPtr = std::unique_ptr<FramedBoxedLocation>;
    using FramedLocationPtr = std::unique_ptr<FramedLocation>;
    using LocationPtr = std::unique_ptr<Location>;

} // namespace armarx::priorknowledge::util
