#pragma once

#include <filesystem>

// Base Class
#include "Entity.h"
#include "detail/ProviderSegmentBase.h"
#include "detail/mixins/DiskStorageMixin.h"
#include "detail/mixins/MongoDBStorageMixin.h"

namespace armarx::armem::server::ltm
{
    class ProviderSegment :
        public detail::ProviderSegmentBase<Entity>,
        public detail::mixin::DiskMemoryItemMixin,
        public detail::mixin::MongoDBStorageMixin
    {
    public:
        ProviderSegment(const detail::mixin::Path&,
                        const detail::mixin::MongoDBSettings&,
                        const std::string&,
                        const MemoryID& id,
                        const std::shared_ptr<Processors>&);

        bool forEachEntity(std::function<void(Entity&)> func) const override;
        bool hasEntity(const std::string&) const override;
        std::shared_ptr<Entity> findEntity(const std::string&) const override;

    protected:
        void _loadAllReferences(armem::wm::ProviderSegment&) override;
        void _loadLatestNReferences(int n, armem::wm::ProviderSegment& p) override;
        void _resolve(armem::wm::ProviderSegment&) override;
        void _store(const armem::wm::ProviderSegment&, bool simulatedVersion) override;

    private:
    };

} // namespace armarx::armem::server::ltm
