#pragma once

// Base Class
#include "../Filter.h"

namespace armarx::armem::server::ltm::processor::filter
{
    enum TimestampType{
        NOT_SET = 0,
        SENT = 1,
        ARRIVED = 2,
        REFERENCED = 3,
        ALL_CORRUPTED = 4
    };

    class SnapshotFrequencyFilter : public SnapshotFilter
    {
    public:
        static const constexpr char* NAME = "SnapshotFrequencyFilter";
        static const constexpr char* PARAM_WAITING_TIME = "WaitingTimeInMsForFilter";

        SnapshotFrequencyFilter() = default;

        virtual bool accept(const armem::wm::EntitySnapshot &e, bool simulatedVersion) override;
        void configure(const nlohmann::json& json) override;

        FilterStatistics getFilterStatistics() override;
        std::string getName() override;

    private:
        void setNonCorruptedTimestampType(const armem::wm::EntitySnapshot &e);
        int getNonCorruptedTimestamp(const armem::wm::EntityInstance &i, bool simulatedVersion);

        std::map<MemoryID, std::int64_t> lastTimesPerEntity;
        int maxDifference = 0;
        bool corruptedWarningGiven = false;
        TimestampType nonCorruptedType = TimestampType::NOT_SET;

    };
} // namespace armarx::armem::server::ltm::processor::filter
