/*
 * This file is part of ArmarX.
 *
 * ArmarX is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * ArmarX is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 *
 * @author     Fabian Reister ( fabian dot reister at kit dot edu )
 * @date       2021
 * @copyright  http://www.gnu.org/licenses/gpl-2.0.txt
 *             GNU General Public License
 */

#pragma once

#include <RobotAPI/libraries/armem_robot_state/types.h>


// #include <RobotAPI/libraries/armem_objects/aron/ObjectClass.aron.generated.h>
// #include <RobotAPI/libraries/armem_objects/aron/ObjectInstance.aron.generated.h>
// #include <RobotAPI/libraries/ArmarXObjects/aron/ObjectPose.aron.generated.h>

#include <RobotAPI/libraries/armem_robot_state/aron/Robot.aron.generated.h>
#include <RobotAPI/libraries/armem_robot_state/aron/RobotDescription.aron.generated.h>
#include <RobotAPI/libraries/armem_robot_state/aron/RobotState.aron.generated.h>

namespace armarx::armem
{
    namespace robot_state::localization
    {
        struct Transform;
        struct TransformHeader;

    } // namespace robot_state::localization

    namespace robot_state
    {
        struct JointState;

    } // namespace robot_state

    namespace prop::arondto
    {
        struct Platform;
        struct ForceTorque;
    } // namespace prop::arondto

    namespace exteroception::arondto
    {
        struct ToF;
    }

    namespace arondto
    {
        struct Transform;
        struct TransformHeader;

        struct JointState;

    } // namespace arondto

} // namespace armarx::armem

namespace armarx::armem::robot_state::localization
{

    void fromAron(const arondto::Transform& dto, Transform& bo);
    void toAron(arondto::Transform& dto, const Transform& bo);

    void fromAron(const arondto::TransformHeader& dto, TransformHeader& bo);
    void toAron(arondto::TransformHeader& dto, const TransformHeader& bo);
} // namespace armarx::armem::robot_state::localization

namespace armarx::armem::robot_state
{

    void fromAron(const armarx::armem::prop::arondto::Platform& dto, PlatformState& bo);
    void toAron(armarx::armem::prop::arondto::Platform& dto, const PlatformState& bo);

} // namespace armarx::armem::robot_state

namespace armarx::armem::robot_state::proprioception
{

    void fromAron(const armarx::armem::prop::arondto::ForceTorque& dto, ForceTorque& bo);
    void toAron(armarx::armem::prop::arondto::ForceTorque& dto, const ForceTorque& bo);

} // namespace armarx::armem::robot_state::proprioception

namespace armarx::armem::robot_state::exteroception
{
    void fromAron(const armarx::armem::exteroception::arondto::ToF& dto, ToF& bo);
    void toAron(armarx::armem::exteroception::arondto::ToF& dto, const ToF& bo);


} // namespace armarx::armem::robot_state::exteroception

namespace armarx::armem::robot_state::description
{
    void fromAron(const arondto::RobotDescription& dto, RobotDescription& bo);
    void toAron(arondto::RobotDescription& dto, const RobotDescription& bo);

} // namespace armarx::armem::robot_state::description

namespace armarx::armem::robot_state
{
    // TODO move the following
    void fromAron(const long& dto, IceUtil::Time& time);
    void toAron(long& dto, const IceUtil::Time& time);
    // end TODO

    void fromAron(const arondto::Robot& dto, Robot& bo);
    void toAron(arondto::Robot& dto, const Robot& bo);

    void fromAron(const arondto::RobotState& dto, RobotState& bo);
    void toAron(arondto::RobotState& dto, const RobotState& bo);

} // namespace armarx::armem::robot_state
