#pragma once

#include <Eigen/Core>
#include <Eigen/Geometry>

#include <RobotAPI/libraries/aron/core/aron_conversions.h>

namespace armarx::aron::eigen
{
    using AronPosef = Eigen::Matrix<float, 4, 4, Eigen::RowMajor>;
    using AronPositionf = Eigen::Matrix<float, 3, 1, Eigen::RowMajor>;
    using AronOrientationf = Eigen::Quaternion<float>;

    template <class EigenT, int rows, int cols>
    void
    fromAron(const Eigen::Matrix<EigenT, rows, cols, Eigen::RowMajor>& dto,
             Eigen::Matrix<EigenT, rows, cols>& bo)
    {
        bo = dto;
    }

    template <class EigenT, int rows, int cols>
    void
    toAron(Eigen::Matrix<EigenT, rows, cols, Eigen::RowMajor>& dto,
           const Eigen::Matrix<EigenT, rows, cols>& bo)
    {
        dto = bo;
    }

    void fromAron(const AronPosef& dto, Eigen::Affine3f& bo);
    void toAron(AronPosef& dto, const Eigen::Affine3f& bo);

    void fromAron(const AronPosef& dto, Eigen::Isometry3f& bo);
    void toAron(AronPosef& dto, const Eigen::Isometry3f& bo);

    void fromAron(const AronPosef& dto, Eigen::Isometry3d& bo);
    void toAron(AronPosef& dto, const Eigen::Isometry3d& bo);

} // namespace armarx::aron::eigen

namespace armarx::aron
{
    using eigen::fromAron;
    using eigen::toAron;
} // namespace armarx::aron

namespace armarx
{
    using aron::eigen::fromAron;
    using aron::eigen::toAron;
} // namespace armarx
