#include "Robot.h"

namespace armarx::robot_name_service::core
{

    void
    TCPInfo::fromIce(const dto::TCPInfo& r)
    {
        name = r.name;
        ft_name = r.ft_name;
        handUnitInterface = r.handUnitInterface;
    }

    dto::TCPInfo
    TCPInfo::toIce() const
    {
        dto::TCPInfo r;
        r.name = name;
        r.ft_name = ft_name;
        r.handUnitInterface = handUnitInterface;
        return r;
    }

    void
    ArmInfo::fromIce(const dto::ArmInfo& r)
    {
        kinematicChainName = r.kinematicChainName;
        hand.fromIce(r.tcpInfo);
    }

    dto::ArmInfo
    ArmInfo::toIce() const
    {
        dto::ArmInfo r;
        r.kinematicChainName = kinematicChainName;
        r.tcpInfo = hand.toIce();
        return r;
    }

    void
    RobotInfo::fromIce(const dto::RobotInfo& r)
    {
        name = r.name;
        xmlPackagePath = r.xmlPackagePath;
        memoryNameSystem = armarx::armem::client::MemoryNameSystem(r.memoryNameSystem);
        skillManager = r.skillManager;

        armInfos.clear();
        for (const auto& [k, a] : r.armInfos)
        {
            armInfos[k].fromIce(a);
        }

        robotUnit = r.robotUnit;
    }

    dto::RobotInfo
    RobotInfo::toIce() const
    {
        dto::RobotInfo r;
        r.name = name;
        r.xmlPackagePath = xmlPackagePath;
        r.memoryNameSystem = memoryNameSystem.getMemoryNameSystem();
        r.skillManager = skillManager;

        for (const auto& [k, a] : armInfos)
        {
            r.armInfos[k] = a.toIce();
        }

        r.robotUnit = robotUnit;
        return r;
    }

} // namespace armarx::robot_name_service::core
