#pragma once

#include <string>

#include <SimoxUtility/algorithm/string.h>

#include <RobotAPI/interface/skills/SkillManagerInterface.h>

#include "ProviderID.h"

namespace armarx
{
    namespace skills
    {
        class SkillID
        {
        public:
            static const constexpr char* NAME_SEPARATOR = "/";
            static const constexpr char* UNKNOWN = "UNKNOWN";

            bool operator==(const SkillID& other) const;
            bool operator!=(const SkillID& other) const;
            bool operator<(const SkillID& other) const;
            bool operator<=(const SkillID& other) const;

            bool
            isFullySpecified() const
            {
                return isSkillSpecified() and isProviderSpecified();
            }

            bool
            isSkillSpecified() const
            {
                return not skillName.empty();
            }

            bool
            isProviderSpecified() const
            {
                if (not providerId.has_value())
                {
                    return false;
                }
                return not providerId->providerName.empty();
            }

            manager::dto::SkillID toManagerIce() const;
            provider::dto::SkillID toProviderIce() const;

            static SkillID FromIce(const manager::dto::SkillID&);
            static SkillID FromIce(const provider::dto::SkillID&,
                                   const std::optional<ProviderID>& providerId = std::nullopt);

            std::string toString() const;

            std::optional<ProviderID> providerId = std::nullopt;
            std::string skillName;
        };

        std::ostream& operator<<(std::ostream& os, const SkillID& id);
    } // namespace skills
} // namespace armarx
