#ifndef SKILLEXECUTIONTREEWIDGET_H
#define SKILLEXECUTIONTREEWIDGET_H

#include <QTreeWidget>

#include <ArmarXCore/core/time.h>

#include "../memory/MemoryCommunicatorBase.h"

namespace armarx::skills::gui
{
    using StatusMap = std::map<skills::SkillExecutionID, skills::SkillStatusUpdate>;

    class SkillExecutionTreeWidget : public QTreeWidget, public MemoryCommunicatorBase
    {
    public:
        SkillExecutionTreeWidget(std::shared_ptr<SkillManagerWrapper> _memory,
                                 QWidget* parent = nullptr) :
            QTreeWidget(parent), MemoryCommunicatorBase(_memory)
        {
            setupUi();
        }

        struct SelectedExecution
        {
            skills::SkillExecutionID skillExecutionId;

            // make default constructable
            SelectedExecution() :
                skillExecutionId{
                    .skillId = {.providerId = std::nullopt, .skillName = skills::SkillID::UNKNOWN},
                    .executorName = skills::SkillExecutionID::UNKNOWN,
                    .executionStartedTime = armarx::core::time::DateTime::Invalid()}
            {
            }
        };

        SelectedExecution& getSelectedExecution();

    public slots:
        void disconnectGui();
        void updateGui(SkillManagerWrapper::Snapshot update);

    private slots:
        void executionSelectionChanged(QTreeWidgetItem* current, QTreeWidgetItem* previous);
        void runContextMenu(const QPoint& pos);
        void stopSelectedExecution();
        void rerunSkillWithSimilarParams();

    private:
        void setupUi();
        void connectSignals();

        /**
         * @brief Checks the validity of the selected item at run time.
         * @return the validity.
         */
        bool selectionValid();
        SelectedExecution selectedExecution;
    };
} // namespace armarx::skills::gui

#endif // SKILLEXECUTIONTREEWIDGET_H
